@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -10,26 +10,7 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-
-#include <sys/types.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <string.h>
-#include <signal.h>
-#include <time.h>
-
-#ifdef AFS_NT40_ENV
-#include <winsock2.h>
-#else
-#include <sys/file.h>
-#include <sys/param.h>
-#include <netinet/in.h>
-#include <netdb.h>
-#endif
-
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
+#include <roken.h>
 
 #include <lock.h>
 #include <rx/xdr.h>
@@ -44,10 +25,10 @@
 static_inline int afs_cast_time_t(time_t d) { return (int) d; }
 
 static short
-PortNumber(register char *aport)
+PortNumber(char *aport)
 {
-    register int tc;
-    register afs_int32 total;
+    int tc;
+    afs_int32 total;
 
     total = 0;
     while ((tc = *aport++)) {
@@ -88,7 +69,7 @@ CommandProc(struct cmd_syndesc *as, void *arock)
 {
     char *hostName, *portName, *times;
     afs_int32 hostAddr;
-    register afs_int32 i, j, code;
+    afs_int32 i, j, code;
     short port;
     int int32p;
     time_t now, then, diff, newtime;
@@ -99,6 +80,7 @@ CommandProc(struct cmd_syndesc *as, void *arock)
     struct ubik_sdebug usdebug;
     int oldServer = 0;		/* are we talking to a pre 3.5 server? */
     afs_int32 isClone = 0;
+    char hoststr[16];
 
     int32p = (as->parms[2].items ? 1 : 0);
 
@@ -164,10 +146,10 @@ CommandProc(struct cmd_syndesc *as, void *arock)
 	printf("Host's addresses are: ");
 	for (j = 0; udebug.interfaceAddr[j] && (j < UBIK_MAX_INTERFACE_ADDR);
 	     j++)
-	    printf("%s ", afs_inet_ntoa(htonl(udebug.interfaceAddr[j])));
+	    printf("%s ", afs_inet_ntoa_r(htonl(udebug.interfaceAddr[j]), hoststr));
 	printf("\n");
     }
-    printf("Host's %s time is %s\n", afs_inet_ntoa(hostAddr), times);
+    printf("Host's %s time is %s\n", afs_inet_ntoa_r(hostAddr, hoststr), times);
 
     times = ctime(&now);
     times[24] = 0;
@@ -192,7 +174,7 @@ CommandProc(struct cmd_syndesc *as, void *arock)
     } else {
 	diff = udebug.now - udebug.lastYesTime;
 	printf("Last yes vote for %s was %d secs ago (%ssync site); \n",
-	       afs_inet_ntoa(htonl(udebug.lastYesHost)),
+	       afs_inet_ntoa_r(htonl(udebug.lastYesHost), hoststr),
 	       afs_cast_time_t(diff),
 	       ((udebug.lastYesState) ? "" : "not "));
 
@@ -200,7 +182,7 @@ CommandProc(struct cmd_syndesc *as, void *arock)
 	newtime = now - diff;
 	times = ctime(&newtime);
 	times[24] = 0;
-	printf("Last vote started %d secs ago (at %s)\n", 
+	printf("Last vote started %d secs ago (at %s)\n",
 	       afs_cast_time_t(diff), times);
     }
 
@@ -222,10 +204,6 @@ CommandProc(struct cmd_syndesc *as, void *arock)
 		 ((udebug.nServers > 1) ? "s" : ""));
 	}
 	printf("Recovery state %x\n", udebug.recoveryState);
-	if (udebug.activeWrite) {
-	    printf("I am currently managing write trans %d.%d\n",
-		   udebug.epochTime, udebug.tidCounter);
-	}
     } else {
 	if (isClone)
 	    printf("I am a clone and never can become sync site\n");
@@ -233,15 +211,23 @@ CommandProc(struct cmd_syndesc *as, void *arock)
 	    printf("I am not sync site\n");
 	diff = udebug.now - udebug.lowestTime;
 	printf("Lowest host %s was set %d secs ago\n",
-	       afs_inet_ntoa(htonl(udebug.lowestHost)),
+	       afs_inet_ntoa_r(htonl(udebug.lowestHost), hoststr),
 	       afs_cast_time_t(diff));
 
 	diff = udebug.now - udebug.syncTime;
 	printf("Sync host %s was set %d secs ago\n",
-	       afs_inet_ntoa(htonl(udebug.syncHost)),
+	       afs_inet_ntoa_r(htonl(udebug.syncHost), hoststr),
 	       afs_cast_time_t(diff));
     }
 
+    if (udebug.activeWrite) {
+	printf("I am currently managing write trans %d.%d\n",
+	       udebug.epochTime, udebug.tidCounter);
+    } else {
+	printf("The last trans I handled was %d.%d\n",
+	       udebug.epochTime, udebug.tidCounter);
+    }
+
     printf("Sync site's db version is %d.%d\n", udebug.syncVersion.epoch,
 	   udebug.syncVersion.counter);
     printf("%d locked pages, %d of them for write\n", udebug.lockedPages,
@@ -289,11 +275,11 @@ CommandProc(struct cmd_syndesc *as, void *arock)
 		break;
 	    }
 	    /* otherwise print the structure */
-	    printf("\nServer (%s", afs_inet_ntoa(htonl(usdebug.addr)));
+	    printf("\nServer (%s", afs_inet_ntoa_r(htonl(usdebug.addr), hoststr));
 	    for (j = 0;
 		 ((usdebug.altAddr[j]) && (j < UBIK_MAX_INTERFACE_ADDR - 1));
 		 j++)
-		printf(" %s", afs_inet_ntoa(htonl(usdebug.altAddr[j])));
+		printf(" %s", afs_inet_ntoa_r(htonl(usdebug.altAddr[j]), hoststr));
 	    printf("): (db %d.%d)", usdebug.remoteVersion.epoch,
 		   usdebug.remoteVersion.counter);
 	    if (isClone)
@@ -307,7 +293,7 @@ CommandProc(struct cmd_syndesc *as, void *arock)
 		newtime = now - diff;
 		times = ctime(&newtime);
 		times[24] = 0;
-		printf("    last vote rcvd %d secs ago (at %s),\n", 
+		printf("    last vote rcvd %d secs ago (at %s),\n",
 		       afs_cast_time_t(diff),
 		       times);
 	    }
@@ -340,8 +326,8 @@ main(int argc, char **argv)
 
 #ifdef	AFS_AIX32_ENV
     /*
-     * The following signal action for AIX is necessary so that in case of a 
-     * crash (i.e. core is generated) we can include the user's data section 
+     * The following signal action for AIX is necessary so that in case of a
+     * crash (i.e. core is generated) we can include the user's data section
      * in the core dump. Unfortunately, by default, only a partial core is
      * generated which, in many cases, isn't too useful.
      */
